//------------------------------------------------------------------------------
// AnimationMaster class header.
//
// Controls the animation of its SoSwitch children.  Cycles the SoSwitch nodes
// through their children once at a given speed and deletes them when they are
// through.
//------------------------------------------------------------------------------

#ifndef	_ANIMATIONMASTER_H_
#define	_ANIMATIONMASTER_H_

#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFTime.h>
#include <Inventor/nodes/SoGroup.h>

class SoFieldSensor ;
class SoSensor ;

class DoubleList : public SbPList {
    public :
	DoubleList() : SbPList() {}
	~DoubleList() { truncate( 0 ) ; }
	double operator[]( int i ) const
	    { return( *(double *)( (* (const SbPList *)this )[i] ) ) ; }
	void append( double v ) ;
	void insert( double v, int addBefore ) ;
	void remove( int which ) ;
	void truncate( int start ) ;
} ;


class AnimationMaster : public SoGroup {

	SO_NODE_HEADER( AnimationMaster ) ;

    public :
	AnimationMaster() ;
	AnimationMaster( int nChildren ) ;
	void		addChild( SoNode *child ) ;
	void		insertChild( SoNode *child, int newChildIndex ) ;
	void		replaceChild( int index, SoNode *newChild ) ;
	void		removeChild( int index ) ;
	static void	initClass() ;

	SoSFFloat	speed ;

    protected :
	~AnimationMaster() ;
	SoSFTime	timeIn ;
	SoFieldSensor	*timeSensor ;
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	virtual SbBool	readInstance( SoInput *in, unsigned short flags ) ;
#else
	virtual SbBool	readInstance( SoInput *in ) ;
#endif
	virtual SbBool	readChildren(SoInput *in) ;
	static void	timeChangedCB( void *data, SoSensor *sensor ) ;

    private :
	void		updateChildren() ;
	void		checkSensor() ;
	double		currentTime ;
	DoubleList	childBirthTime ;
	void		buildCommon() ;
	SbBool		typeOk( SoNode *node ) ;
} ;

#endif	/* _ANIMATIONMASTER_H_ */
