//------------------------------------------------------------------------------
// Abstract base class header for enemy robots in Pointblank.
//------------------------------------------------------------------------------

#ifndef _AUTOMATON_H_
#define _AUTOMATON_H_

#include <Inventor/fields/SoSFBool.h>
#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFInt32.h>
#include "Missile.h"
#include "Obstacle.h"

class Vehicle ;
class Missile ;

typedef void AutomatonStrikeHandler( Missile *mis, SoNode *target,
					void *userData ) ;
typedef Missile *AutomatonLaunchHandler( SoType mType, void *userData ) ;



class Automaton : public Obstacle {

	SO_NODE_ABSTRACT_HEADER( Automaton ) ;

    public :
	enum StatusFlags {
		ALIVE,		// Fully active
		DYING,		// Destroyed, but has active missiles
		DEAD		// Destroyed with no active missiles
		} ;
	static void	initClass() ;
	SoSFFloat	skill ;
	SoSFFloat	armor ;
	SoSFInt32	value ;
	SoSFBool	alive ;
	SoSFFloat	rangeOfInterest ;
	SbBool		isAlive() { return( status == ALIVE ) ; }
	void		update( float deltaTime, SoNode *scene,
				Vehicle *player ) ;
	int32_t		sufferDamage( float amount ) ;
	virtual void	reset() = 0 ;
	static void	setStrikeHandler( AutomatonStrikeHandler *func,
				void *userData ) ;
	static void	setLaunchHandler( AutomatonLaunchHandler *func,
				void *userData ) ;

    protected :
	Automaton() ;
	Automaton( const char *name ) ;
	~Automaton() ;
	StatusFlags	status ;
	float		damage ;
	virtual void	updateSelf( float deltaTime, SoNode *scene,
				Vehicle *player ) = 0 ;
	virtual void	die() = 0 ;
	Missile		*launchMissile( SoType type )
				{ return(launchHandler(type,launchData)); }
	SoNodeList	missileList ;
	static AutomatonStrikeHandler	*strikeHandler ;
	static void			*strikeData ;
	static AutomatonLaunchHandler	*launchHandler ;
	static void			*launchData ;

    private :
	void	createIt() ;
	static void	defaultStrikeHandler( Missile *, SoNode *, void * );
	static Missile	*defaultLaunchHandler( SoType mType, void * );
	Automaton( const Automaton &a ) ;
	Missile::MissileStatus	guideMissile( Missile *m, float dt,
					SoNode *scene, Vehicle *target ) ;
} ;
#endif	/* !_AUTOMATON_H_ */
