//------------------------------------------------------------------------------
// Non-mobile automaton.
//------------------------------------------------------------------------------

#ifndef	_BATTERY_H_
#define	_BATTERY_H_

#include <Inventor/fields/SoSFName.h>
#include "Automaton.h"

class SoGroup ;
class SoSwitch ;
class SoRotation ;
class SoFieldSensor ;


class Battery : public Automaton {

	SO_NODE_HEADER( Battery ) ;

    public :
	static void	initClass() ;
	Battery() ;
	Battery( const char *name ) ;
	SoSFFloat	minAzim ;
	SoSFFloat	maxAzim ;
	SoSFFloat	minIncl ;
	SoSFFloat	maxIncl ;
	SoSFName	typeOfMissile ;
	SoSFFloat	reloadTime ;
	virtual void	reset() ;

    SoEXTENDER public :
	virtual void	collide( CollisionAction *action ) ;
	virtual void	obstruct( LosAction *action ) ;

    protected :
	~Battery() ;
	enum TrackingStyle {
		DEAD_ON,
		AZIM_ONLY,
		GUIDE,
		LOB,
		} ;
	SoSwitch	*view ;
	SoRotation	*inclRot ;
	SoRotation	*azimRot ;
	SoType		missileType ;
	SbVec3f		launchPoint ;
	float		timeToFire ;
	virtual void	processGeometry() ;
	virtual SbBool	postRead( SbBool status ) ;
	TrackingStyle	tracking ;
	float		incl ;
	float		azim ;
	SbRotation	aimAngle ;
	SbVec3f		aimAhead ;
	SbVec3f		aimRight ;
	SbVec3f		aimUp ;
	SoFieldSensor	*minAzimSensor ;
	SoFieldSensor	*maxAzimSensor ;
	SoFieldSensor	*minInclSensor ;
	SoFieldSensor	*maxInclSensor ;
	void		fire() ;
	SbBool		aim( float dt, Vehicle *player ) ;
	void		setRotations() ;
	virtual void	updateSelf( float deltaTime, SoNode *scene,
				Vehicle *player ) ;
	virtual void	die() ;
	virtual void	enableSensors() ;
	virtual void	disableSensors() ;
	void		adjustAzim( float dt, float da, float dr ) ;
	void		adjustIncl( float dt, float da, float du ) ;

    private :
	void		createIt() ;
	static void	checkAnglesCB( void *userData, SoSensor *sensor ) ;
	void		checkAngles() ;
	SbBool		checkLimits() ;
	float		maxAzimuth ;
	float		maxInclination ;
	float		missileSpeed ;
} ;

#endif	/* !_BATTERY_H_ */
