//------------------------------------------------------------------------------
// Pointblank render area class.  Controls multiple scene graphs.
//------------------------------------------------------------------------------

#ifndef _COCKPIT_H_
#define _COCKPIT_H_

#include <Inventor/Xt/SoXtGLWidget.h>
#include <Inventor/SbPList.h>
#include <Inventor/actions/SoGLRenderAction.h>

class SbColor ;
class SoXtMouse ;
class SoXtKeyboard ;
class SoSceneManager ;

typedef SbBool	CockPitEventCB( void *userData, XAnyEvent *anyEvent ) ;

class CockPit : public SoXtGLWidget {

    public :
	CockPit( Widget parent = NULL, const char *name = NULL ) ;
	~CockPit() ;

	void	setSceneBackgroundColor( const SbColor &c, int sceneNo ) ;
	int	addScene( SoNode *newScene, float left, float right,
			float bottom, float top, SbBool clearWindow ) ;
	void	clearScenes() ;
	void	setEventCallback( CockPitEventCB *func, void *userData = NULL )
			{ appHandler = func ; appHandlerData = userData ; }
	void	render() ;
	void	renderNoSwap() ;
	void	setOverlaySceneGraph( SoNode *scene )  ;
	void	setTransparencyType( SoGLRenderAction::TransparencyType type )
		    { ra->setTransparencyType( type ) ; }
	SoGLRenderAction::TransparencyType	getTransparencyType()
		    { return( ra->getTransparencyType() ) ; }
	const SbVec2s &getSceneCenter( int sceneNo ) const ;
	const SbVec2s &getSceneSize( int sceneNo ) const ;
	const SbViewportRegion &getSceneViewportRegion( int sceneNo ) ;
	SbBool hasFastTextures() const { return( useTexture ) ; }
	void drainEvents() ;
	void forceRedraw() ;

    protected :
	virtual void	redraw() ;
	virtual void	afterRealizeHook() ;
	virtual void	sizeChanged( const SbVec2s &newSize ) ;
	virtual void	processEvent( XAnyEvent *anyEvent ) ;
	virtual void	initGraphic() ;
	virtual void	initOverlayGraphic() ;
	SoGLRenderAction	*ra ;
	SbBool		useTexture ;

    private :
	SbPList	sceneList ;
	int	needWindowClear ;
	CockPitEventCB	*appHandler ;
	void		*appHandlerData ;
	SoXtMouse	*mouseDevice ;
	SoXtKeyboard	*keyboardDevice ;
	SoSceneManager	*overlayManager ;
	void		redrawOverlay() ;
	XColor		*overlayMapColors ;
	static void	windowEventCB( Widget, CockPit *, XAnyEvent *,
				Boolean * ) ;
	static void	renderOverlayCB( void *data, SoSceneManager *mgr ) ;
	SbBool		draining ;
} ;

#endif /* _COCKPIT_H_ */
