//------------------------------------------------------------------------------
// Debris class.
//------------------------------------------------------------------------------

#include "FollowTerrainAction.h"
#include "Constants.h"
#include "Debris.h"

SO_NODE_SOURCE( Debris ) ;



//------------------------------------------------------------------------------
// Initialize the class.
//------------------------------------------------------------------------------
void
Debris::initClass()
{
	SO_NODE_INIT_CLASS( Debris, TransformSwitch, "TransformSwitch" ) ;
}



//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
Debris::Debris()
{
	SO_NODE_CONSTRUCTOR( Debris ) ;
	SO_NODE_ADD_FIELD( value, ( 10.0f ) ) ;
	onGround = FALSE ;
	fta = new FollowTerrainAction ;
	fallSpeed = 0 ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
Debris::~Debris()
{
	delete fta ;
}



//------------------------------------------------------------------------------
// Update.  If the object isn't on the ground, then drop it.
//------------------------------------------------------------------------------
void
Debris::update(
	float	dt,
	SoNode	*scene
	)
{
	if( onGround )
	{
		return ;
	}

	SbVec3f pos = position.getValue() ;

	fta->setSearchPoint( pos ) ;
	fta->apply( scene ) ;

	const GroundPointList &gpl = fta->getGroundPointList() ;

	int nHits = gpl.getLength() ;
	if( nHits > 0 )
	{
		int n = 0 ;
		while( ( n < nHits - 1 ) && gpl[n]->getHeight() > 0.0f )
		{
			n++ ;
		}

		if( gpl[n]->getHeight() > dt * fallSpeed )
		{
			position.setValue( gpl[n]->getPoint() ) ;
			SbVec3f up = gpl[n]->getNormal() ;
			up.normalize() ;
			SbVec3f right( 1.0f, 0.0f, 0.0f ) ;
			SbVec3f &ahead = up.cross( right ) ;
			if( ahead.normalize() < 0.1f )
			{
				right.setValue( 0.0f, 1.0f, 0.0f ) ;
				ahead = up.cross( right ) ;
				ahead.normalize() ;
			}
			right = ahead.cross( up ) ;
			right.normalize() ;

			SbMatrix mat( right[0], right[1], right[2], 0.0f,
					ahead[0], ahead[1], ahead[2], 0.0f,
					up[0], up[1], up[2], 0.0f,
					0.0f, 0.0f, 0.0f, 1.0f ) ;

			SbRotation newRot( mat ) ;
			rotation.setValue( newRot ) ;
			onGround = TRUE ;
			return ;
		}

		fallSpeed -= dt * Constants::Gravity ;

		pos[2] += dt * fallSpeed ;
		position.setValue( pos ) ;
	}
	else
	{
		onGround = TRUE ;
	}
}
