//------------------------------------------------------------------------------
// Class header for FollowTerrainAction
//------------------------------------------------------------------------------

#ifndef _FOLLOWTERRAINACTION_H_
#define _FOLLOWTERRAINACTION_H_

#include <Inventor/actions/SoSubAction.h>
#include <Inventor/SbLinear.h>

class GroundPoint {
    public :
	GroundPoint( const SbVec3f &point, const SbVec3f &normal,
		const SbVec3f &seachPoint ) ;
	~GroundPoint() ;
	const SbVec3f &getPoint() const { return point ; } ;
	const SbVec3f &getNormal() const { return normal ; } ;
	//
	// Return the height of the point above the search point.
	//
	float getHeight() const { return height ; } ;
	float getDistance() const { return distance ; } ;
	void setPoly( int nPts, const SbVec3f *pts ) ;
	const SbVec3f *getPoly( int &nPts )
		{ nPts = numPoints ; return polyPt ; }
    private :
	GroundPoint() ;
	SbVec3f	point ;
	SbVec3f	normal ;
	SbVec3f polyPt[4] ;
	int	numPoints ;
	float	height ;
	float	distance ;
} ;


class GroundPointList : public SbPList {
    public :
	GroundPointList() : SbPList() {}
	~GroundPointList() { truncate( 0 ) ; }
	GroundPoint *operator []( int i ) const
	    { return ( (GroundPoint *) ( (* (const SbPList *)this )[i] ) ) ; } ;
	void insert( GroundPoint *gp, int addBefore )
	    { SbPList::insert( (void *)gp, addBefore ) ; }
	void remove( int which ) ;
	void truncate( int start ) ;
} ;


class FollowTerrainAction : public SoAction {

	SO_ACTION_HEADER( FollowTerrainAction ) ;

    public :
	static void initClass() ;
	FollowTerrainAction() ;
	virtual ~FollowTerrainAction() ;
	void setSearchPoint( const SbVec3f &worldSpacePoint ) ;
	const SbVec3f &getSearchPoint() const { return searchPoint ; } ;
	GroundPoint &addIntersection( const SbVec3f &worldSpacePoint,
				const SbVec3f &normal ) ;
	const GroundPointList &getGroundPointList() { return iList ; }

    protected :
	virtual void beginTraversal( SoNode *node ) ;

    private :
	static void callDoAction( SoAction *, SoNode * ) ;
	static void doNada( SoAction *, SoNode * ) ;
	SbVec3f	searchPoint ;
	GroundPointList	iList ;
} ;

#endif /* !_FOLLOWTERRAINACTION_H_ */
