//------------------------------------------------------------------------------
// Game Control class.
//------------------------------------------------------------------------------

#ifndef	_GAMECONTROL_H_
#define	_GAMECONTROL_H_

#include <Inventor/SbString.h>
#include <Inventor/SbPList.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>

#include "WeaponControl.h"

class Automaton ;
class Bomb ;
class CockPit ;
class DamageControl ;
class Dashboard ;
class DebrisControl ;
class ExplosionControl ;
class GameIO ;
class GameScenario ;
class Goal ;
class InterComm ;
class LosAction ;
class NameList ;
class Obstacle ;
class Player ;
class PlayerPacket ;
class RadarControl ;
class RotateRotation ;
class Sfx ;
class SfxLoop ;
class SfxStereoPitch ;
class SoCalculator ;
class SoCoordinate3 ;
class SoGroup ;
class SoPerspectiveCamera ;
class SoSeparator ;
class SoSwitch ;
class SoRotor ;
class SoText3 ;
class SoTransformVec3f ;
class Vehicle ;
class SoRayPickAction ;
class Speedometer ;


class PlayerList : public SbPList {

    public :
	PlayerList() : SbPList() {}
	~PlayerList() { truncate( 0 ) ; }
	Player *operator[]( int i ) const
		{ return( (Player *)( (* (const SbPList *)this )[i] ) ) ; }
	void append( Player *player )
		{ SbPList::append( (void *)player ) ; }
	void truncate( int start ) ;
	int find( HostAddr remoteId ) ;
	void remove( int which ) ;
} ;



class GameControl {

    public :

	enum Views {
		COCKPIT, AHEAD, BEHIND, LEFT, RIGHT, OVERHEAD, UNDER,
		LEFT_REAR, RIGHT_FRONT, RIGHT_REAR, LEFT_FRONT,
		INC_DISTANCE, DEC_DISTANCE, NEGATIVE_X, NEGATIVE_Y,
		POSITIVE_Z, MISSILE, LAST_MODE,
		} ;
	enum { MAX_TEAMS = 4 } ;
	enum GameMode {
		SOLO,
		FREE,
		GOAL,
		} ;

	enum PlayerStatus {
		ALIVE			= 0x01,
		DEAD			= 0x02,
		CHOOSING_TANK		= 0x04,
		CHOOSING_TEAM		= 0x08,
		INITIALIZING		= 0x10,
		ROUND_OVER		= 0x20,
		HAS_VEHICLE		= ( ALIVE | DEAD ),
		} ;

	GameControl( CockPit *cp, GameIO *spigot, GameScenario *scenario ) ;
	~GameControl() ;
	void	chooseVehicle() ;
	void	setCameraMode( Views mode ) ;
	void	resetVehicle() ;
	void	buttonEvent( XButtonEvent *buttonEvent ) ;
	void	motionEvent( XMotionEvent *motionEvent ) ;
	SbBool	doOneFrame( float dt ) ;
	SbBool	keyPressEvent( KeySym ks ) ;
	SbBool	keyReleaseEvent( KeySym ks ) ;
	void	step( float dt ) ;
	void	removePlayer( HostAddr remoteId, SbBool callGameIO ) ;
	void	buildCaches() ;
	Dashboard	*dBoard ;

    private :
	void	broadcast( float dt ) ;
	void	updateRadar() ;
	SbBool	loadGame( const char *gamename ) ;
	void	setUpOverlay() ;
	GameControl() ;
	SbBool	checkForCameras() ;
	SbBool	setUpDropZones() ;
	SbBool	findTerrain() ;
	SbBool	getVehicleTypes() ;
	void	setBackgroundColor() ;
	void	setUpCamera() ;
	void	launchMissile() ;
	void	processPlayer( PlayerPacket *pkt ) ;
	void	processInfo( InfoPacket *pkt ) ;
	void	die( HostAddr culprit ) ;
	void	updateStatus() ;
	Player	*addPlayer( PlayerPacket *pkt ) ;
	void	resetMouse() ;
	static void	processPlayerPacket( PlayerPacket *pkt,
				GameControl *game ) ;
	static void	processInfoPacket( InfoPacket *pkt,
				GameControl *game ) ;
	static HostAddr	missileStrike( SoNode *node, Missile *mis,
				GameControl *gc ) ;
	static void	runCommand( void *s ) ;
	HostAddr	checkMissileStrike( SoNode *node, Missile *mis ) ;
	void		setUpVehicleChooser() ;
	void		setUpTeamChooser() ;
	void		pickTank( XButtonEvent *be ) ;
	void		pickTeam( XButtonEvent *be ) ;
	void		startChoosingTank() ;
	void		describeVehicle() ;
	SbBool		checkBombHit( Missile *bomb ) ;
	float		checkBombDamage( Missile *bomb, Obstacle *obst ) ;
	Vehicle		*createVehicle( int typeNo ) ;
	void		removeVehicleFromDisplay( Vehicle *v ) ;
	void		addVehicleToDisplay(  Vehicle *v, int team ) ;
	void		updateTeamCounts() ;
	void		findRange() ;
	static void	droppedHost( HostAddr remoteId, GameControl *game ) ;
	static void	bannedHost( HostAddr remoteId, GameControl *game ) ;
	void		showScores() ;
	void		showHelp() ;
	void		setTeamColors() ;
	void		setTeamColorNode( SoNode *node, int n ) ;

	PlayerList		playerList ;
	SoSeparator		*gameRoot ;
	SoSeparator		*overlayRoot ;
	SoSeparator		*realStuff ;
	SoSeparator		*topRoot ;
	SoSwitch		*modeSwitch ;
	SoSwitch		*vehicleHolder ;
	SoRotor			*vehicleRotor ;
	SoBaseColor		*vehicleChoiceColor ;
	SoText3			*vehicleText ;
	SoText3			*weaponText ;
	SoBaseColor		*teamColors ;
	NameList		*teamNames ;
	SoBaseColor		*radarColors ;
	SoText3			*teamCounts ;
	Vehicle			*vehicle ;
	SoSeparator		*vehicleGroup ;
	SoGroup			*missileGroup ;
	SoCoordinate3		*dropZones ;
	SoPerspectiveCamera	*cam ;
	SoCalculator		*camCalc ;
	SoTransformVec3f	*xfv ;
	RotateRotation		*rr ;
	RadarControl		*radarControl ;
	DamageControl		*damageControl ;
	WeaponControl		*weaponControl ;
	ExplosionControl	*explosionControl ;
	DebrisControl		*debrisControl ;
	NameList		*vehicleChoices ;
	SbString		name ;
	SoSeparator		*litObjects ;
	SoSwitch		*overlaySwitch ;
	SoSwitch		*overlayTextSwitch ;
	SoText3			*overlayText ;
	CockPit			*cockPit ;
	SbBool			noVehicleInput ;
	WeaponControl::WeaponPod	 weaponPod ;
	LosAction		*la ;
	GameIO			*gio ;
	float			gameTime ;
	float			statusTime ;
	PlayerStatus		status ;
	HostAddr		otherHost ;
	float			mouseX ;
	float			mouseY ;
	InterComm		*comm ;
	SbVec2s			mainViewCenter ;
	Sfx			*struckPlayer ;
	Sfx			*dieNoise ;
	Sfx			*hitNoise ;
	Sfx			*buzzer ;
	Sfx			*gong ;
	Sfx			*hello ;
	SfxLoop			*wind ;
	SfxStereoPitch		*engine ;
	unsigned short		vehicleChoice ;
	int			myScore ;
	int			myTeam ;
	float			scavengeTime ;
	SoRayPickAction		*ray ;
	int			mouseWinX ;
	int			mouseWinY ;
	Speedometer		*speedometer ;
	SbBool			scoresOn ;
	SbBool			helpOn ;
	int			numTeams ;
	GameMode		mode ;
	SoText3			*breakMessage ;
	void			setUpBreakMessage() ;
	//
	// Goal stuff
	//
	enum GoalMode { OPEN, STRUCK_GOAL, CLAIMING } ;
	int			goalsStruck ;	// Bit mask.
	int			numGoalsLeft ;	// Count.
	int			myOrigTeam ;
	GoalMode		goalMode ;
	float			goalStrikeTime ;
	float			roundOverTime ;
	int			strikeAnnouncements ;
	int			myStruckGoal ;
	SoGroup			*goalGroup ;
	int			goalTaker[MAX_TEAMS] ;
	int			goalRound ;
	int			teamAssignment[MAX_TEAMS] ;
	int			teamScores[MAX_TEAMS] ;
	void			struckGoal( int goalTeam ) ;
	void			processGoal( GoalPacket *pkt ) ;
	SbBool			setUpGoals() ;
	void			blowUpGoal( int n ) ;
	void			endGoalGame() ;
	int			getGoalsTeam( Goal * ) ;
	SbBool			goalClaimed( int owner, int killer ) ;
	void			initGoals() ;
	void			startNewRound() ;
	void			goalAdmin() ;
	static void		processGoalPacket( GoalPacket *pkt,
							GameControl *game ) ;
	//
	// Solo stuff
	//
	SoNodeList		automatons ;
	SoNodeList		goals ;
	void			setUpSoloGame() ;
	void			endSoloLevel() ;
	void			endSoloGame() ;
	static Missile		*automatonLaunchHandler( SoType, void * ) ;
	static void		automatonStrikeHandler( Missile *, SoNode *,
					void * ) ;
	void			automatonStrike( Missile *, SoNode * ) ;
	SbString		nextSoloLevel ;
	int			level ;
	void			hitAutomaton( Automaton *a, float damage ) ;
	PlayerStatus		lastStatus ;
	int			numTanks ;
	SbVec3f			lastPosition ;
} ;

#endif	/* _GAMECONTROL_H_ */
