//------------------------------------------------------------------------------
// Class header for GameIO.
//------------------------------------------------------------------------------

#ifndef	_GAMEIO_H_
#define	_GAMEIO_H_

#include <Inventor/SbBasic.h>
#include <Inventor/SbPList.h>

#include "GamePipe.h"
#include "GameOutput.h"

typedef void GamePacketHandler( GamePacket *packetBuffer, void *userData ) ;
typedef void DropHostHandler( HostAddr remoteId, void *userData ) ;


class Handler ;
class HostInfo ;
class GameInput ;
class GameOutput ;
class AckPacket ;

class HandlerList : public SbPList {

    public :
	HandlerList() : SbPList() {}
	~HandlerList() { truncate( 0 ) ; }
	Handler *operator[]( int i ) const
		{ return( (Handler *)( (* (const SbPList *)this )[i] ) ) ; }
	void append( Handler *handler )
		{ SbPList::append( (void *)handler ) ; }
	void truncate( int start ) ;
} ;


class HostInfoList : public SbPList {

    public :
	HostInfoList() : SbPList() { out = NULL ; }
	~HostInfoList() { truncate( 0 ) ; }
	SbBool isOkay( HostAddr remoteId ) ;
	HostInfo *operator[]( int i ) const
		{ return( (HostInfo *)( (* (const SbPList *)this )[i] ) ) ; }
	void append( HostInfo *hostInfo )
		{ SbPList::append( (void *)hostInfo ) ; }
	void truncate( int start ) ;
	void setGameOutput( GameOutput *gameOut ) { out = gameOut ; }
	void accept( HostAddr remoteId, const char *name ) ;
	void remove( int which ) ;
	int  find( HostAddr remoteId ) ;
    private :
	GameOutput	*out ;
} ;



class GameIO {

    friend class HostInfo ;

    public :

	enum { MAX_NAME_LEN = 12 } ;

	// NULL for either argument disables input or output.
	GameIO( const char *inFileName, const char *outFileName ) ;
	GameIO( const char *serviceName, const char *groupName,
		const char *interface, int defaultPort, int loopback,
		int timeToLive ) ;
	~GameIO() ;
	void setIdentifier( unsigned short id, unsigned char version ) ;
	static void setMyName( const char *newName ) ;
	static const char *getMyName() { return( myName ) ; }
	int send( GamePacket *packet, int size )
	    { return( out->send( packet, size ) ) ; }
	void processInput() ;
	void removeHost( HostAddr remoteId ) ;
	void banHost( HostAddr remoteId ) ;
	void dropHost( HostAddr remoteId ) ;
	void addInputPacketType( unsigned char token, int size,
				GamePacketHandler *handler, void *userData ) ;
	HostAddr getMyId() const { return( out->getMyAddress() ) ; }
	void setDropHostHandler( DropHostHandler *h, void *userData ) ;
	void setBanHostHandler( DropHostHandler *h, void *userData ) ;
	const char *getPlayerName( HostAddr remoteId ) ;
	static float	getGameTime() { return( gameTime ) ; }
	static void	updateGameTime( float dt ) { gameTime += dt ; }
	static const char *getHostName( HostAddr remoteId ) ;

    private :
	GameIO() ;
	GameIO( const GameOutput &go ) ;
	void reallyAddInputPacketType( unsigned char token, int size,
				GamePacketHandler *handler, void *userData ) ;
	unsigned short	gameIdentifier ;
	unsigned char	gameVersion ;
	HandlerList	handlerList ;
	HostInfoList	hostList ;
	GameInput	*in ;
	GameOutput	*out ;
	void		*buffer ;
	int		bufferSize ;
	void		processAckPacket( AckPacket *pkt ) ;
	static float	gameTime ;
	void		buildCommon() ;
	static DropHostHandler	*dropHandler ;
	static void		*dropData ;
	static DropHostHandler	*banHandler ;
	static void		*banData ;
	static char		myName[MAX_NAME_LEN] ;
} ;

#endif	/* _GAMEIO_H_ */
