//------------------------------------------------------------------------------
// GamePacket declarations and definitions for Pointblank.
//------------------------------------------------------------------------------

#ifndef	_GAMEPACKET_H_
#define	_GAMEPACKET_H_

#include "Constants.h"
#include "Missile.h"
#include "GamePipe.h"


#define	PLAYER_PKT	0x01
#define	INFO_PKT	0x02
#define COMM_PKT	0x03
#define	GOAL_PKT	0x04


class Location {
    public :
	Location() {}
	Location( const float *v ) { setData( v ) ; }
	void setData( const float *v )
	    { pos[0] = v[0] ; pos[1] = v[1] ; pos[2] = v[2] ; }
	float	pos[3] ;
} ;


class Orientation {
    public :
	Orientation() {}
	Orientation( const float *rot ) { setData( rot ) ; }
	void setData( const float *rot )
	    { o[0] = rot[0] ; o[1] = rot[1] ; o[2] = rot[2] ; o[3] = rot[3] ; }
	float	o[4] ;
} ;


class MissileInfo {
    public :
	MissileInfo() {}
	void setData( HostAddr st, Missile::MissileType mType, const float *pos,
		const float *rot )
		{ status = st ; type = mType ; location.setData( pos ) ;
		orientation.setData( rot ) ; }
	//
	// Overload first field of orientation with damage amount since
	// damage amount only matters when the missile has exploded and
	// the orientation is ignored then anyway.
	//
	void setDamage( float amount ) { orientation.o[0] = amount ; }
	HostAddr		status ;
	Missile::MissileType	type ;
	Location		location ;
	Orientation		orientation ;
} ;



class PlayerPacket : public GamePacket {
    public :
	PlayerPacket( unsigned char st, unsigned char teamId,
		unsigned char origTeamId, unsigned char vType, HostAddr host,
		int points, const float *pos, const float *rot ) :
		GamePacket( PLAYER_PKT ),
		status( st ),
		team( teamId ),
		origTeam( origTeamId ),
		vehicleType( vType ),
		remoteHost( host ),
		score( points ),
		location( pos ),
		orientation( rot )
		{}
	HostAddr	missileStatus( int i )
				{ return( missileInfo[i].status ) ; }
	const float	*missileLocation( int i )
				{ return( missileInfo[i].location.pos ) ; }
	const float	*missileOrientation( int i )
				{ return( missileInfo[i].orientation.o ) ; }
	Missile::MissileType missileType( int i )
				{ return( missileInfo[i].type ) ; }
	float		getDamage( int i )
				{ return( missileInfo[i].orientation.o[0] ) ; }
	unsigned short	playerStatus() { return( status ) ; }
	int		getVehicleType() { return( (int)vehicleType ) ; }
	HostAddr	getCulprit() { return( remoteHost ) ; }
	int		getScore() { return( score ) ; }
	int		getTeam() { return( (int)team ) ; }
	int		getOrigTeam() { return( (int)origTeam ) ; }

	unsigned char	status ;
	unsigned char	vehicleType ;
	unsigned char	team ;
	unsigned char	origTeam ;
	HostAddr	remoteHost ;
	int		score ;
	Location	location ;
	Orientation	orientation ;
	MissileInfo	missileInfo[Constants::MAX_WEAPONS] ;
} ;



class InfoPacket : public GamePacket {
    public :
	enum InfoData {
		QUIT_GAME = 0,
		BANNING = 1,
		DROPPING = 2,
		} ;
	InfoPacket( InfoData infoData, HostAddr remoteId ) :
		GamePacket( INFO_PKT ),
		data( infoData ),
		remoteHost( remoteId )
		{}
	InfoData	data ;
	HostAddr	remoteHost ;
} ;



class GoalPacket : public GamePacket {
    public :
	enum GoalData {
		STRIKE = 0,
		CLAIM = 1,
		UPDATE = 2,
		} ;
	GoalPacket( GoalData goalData, int ownerTeam, int killerTeam,
			int theRound, int flags, float theGameTime ) :
		GamePacket( GOAL_PKT ),
		data( goalData ),
		owner( ownerTeam ),
		killer( killerTeam ),
		round( theRound ),
		flagBits( flags ),
		gameTime( theGameTime )
		{}
	GoalData	data ;
	int		owner ;
	int		killer ;
	int		round ;
	int		flagBits ;
	float		gameTime ;
} ;


#endif	/* _GAMEPACKET_H_ */
