//------------------------------------------------------------------------------
// GamePipe base class for GameInput and GameOutput classes.
//------------------------------------------------------------------------------

#include <stdio.h>
#include <netdb.h>
#include <Inventor/errors/SoDebugError.h>
#include "GamePipe.h"


GamePipeErrorHandler	*GamePipe::errorHandler = 0 ;

unsigned short	GamePacket::gameIdentifier = 0 ;
unsigned char	GamePacket::gameVersion = 0 ;

//------------------------------------------------------------------------------
// Handle an error.
//------------------------------------------------------------------------------
void
GamePipe::handleError(
	Severity	s
	)
{
	if( errorHandler )
	{
		errorHandler( s ) ;
	}
}



//------------------------------------------------------------------------------
// Look up the port number from the service name.
//------------------------------------------------------------------------------
int
GamePipe::getPortFromService(
	const char	*serviceName,
	int		defaultPort
	)
{
	struct servent	*serv ;
	int		port ;
	static int	msgPrinted = 0 ;

	if( ( serv = getservbyname( serviceName, NULL ) ) == NULL )
	{
		if( !msgPrinted )
		{
			msgPrinted = 1 ;
			SoDebugError::postWarning( "getPortFromService",
				"\nCan not find udp service \"%s\"\nTo run "
				"over the network, you must have the following "
				"line in your\n/etc/services file:\n\n\t%s\t%d/"
				"udp\n\nUsing default port of %d.", serviceName,
				serviceName, defaultPort, defaultPort ) ;
		}
		port = defaultPort ;
	}
	else
	{
		port = serv->s_port ;
	}

	return( port ) ;
}



//------------------------------------------------------------------------------
// Static method for GamePacket.
//------------------------------------------------------------------------------
void
GamePacket::setGameId(
	unsigned short	id,
	unsigned char	version
	)
{
	gameIdentifier = id ;
	gameVersion = version ;
}
