//------------------------------------------------------------------------------
// GridTracker class header.
//------------------------------------------------------------------------------

#ifndef _GRIDTRACKER_H_
#define _GRIDTRACKER_H_

#include <Inventor/SbLinear.h>

class SoCoordinate3 ;
class SoNormal ;

class GridTracker {

    enum Status {
	OK        = 0x00,	// Point was located in current grid.
	OFF_X_MIN = 0x01,	// Point was off grid in -X direction.
	OFF_X_MAX = 0x02,	// Point was off grid in +X direction.
	OFF_Y_MIN = 0x04,	// Point was off grid in -Y direction.
	OFF_Y_MAX = 0x08,	// Point was off grid in +Y direction.
    } ;

    public :
	GridTracker( SoCoordinate3 *terrainCoords,
		     SoNormal      *terrainNormals,
		     long          verticesPerColumn,
		     long          verticesPerRow ) ;
	~GridTracker() ;
	int track( SbVec3f &desiredPosition, SbVec3f &actualPosition,
		    SbVec3f &actualNormal ) ;

    private :
	GridTracker() ;
	GridTracker( GridTracker & ) ;

	SoCoordinate3 *coords ;
	SoNormal *normals ;
	long vpCol ;
	long vpRow ;
	float minX ;
	float maxX ;
	float minY ;
	float maxY ;
	float deltaX ;
	float deltaY ;
} ;

#endif /* !_GRIDTRACKER_H_ */
