//------------------------------------------------------------------------------
// GroundShape abstract class header.
//------------------------------------------------------------------------------

#ifndef	_GROUNDSHAPE_H_
#define	_GROUNDSHAPE_H_

#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoMFFloat.h>
#include <Inventor/fields/SoMFVec3f.h>
#include <Inventor/misc/SoChildList.h>
#include <Inventor/nodes/SoSubNode.h>

class SoSensor ;
class SoSeparator ;
class SoCoordinate3 ;
class SoTextureCoordinate2 ;
class SoTransform ;
class SoShapeHints ;
class SoFieldSensor ;
class FollowTerrainAction ;
class LosAction ;

class GroundShape : public SoNode {

	SO_NODE_ABSTRACT_HEADER( GroundShape ) ;

    public :
	static void	initClass() ;

	SoSFVec3f	origin ;	// in X-Y plane
	SoSFFloat	rotation ;	// in radians from the X axis
	SoSFFloat	width ;
	SoSFFloat	length ;
	SoSFFloat	repeatS ;
	SoSFFloat	repeatT ;
	SoMFFloat	height ;
	SoMFVec3f	vector ;

	virtual SbBool	affectsState() const ;

    SoEXTENDER public :
	virtual void	doAction( SoAction *action ) ;
	virtual void	getBoundingBox( SoGetBoundingBoxAction *action ) ;
	virtual void	GLRender( SoGLRenderAction *action ) ;
	virtual void	pick( SoPickAction *action ) ;
	virtual void	getMatrix( SoGetMatrixAction *action ) ;
	virtual void	callback( SoCallbackAction *action ) ;
	virtual void	followTerrain( FollowTerrainAction *action ) ;
	virtual void	obstruct( LosAction *action ) ;
	virtual SoChildList *getChildren() const ;
	static void	followTerrainS( SoAction *action, SoNode *node ) ;
	static void	obstructS( SoAction *action, SoNode *node ) ;

    protected:
	GroundShape() ;
	virtual	~GroundShape() ;
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	virtual SbBool	readInstance( SoInput *in, unsigned short flags ) ;
#else
	virtual SbBool	readInstance( SoInput *in ) ;
#endif
	virtual void	buildIt() = 0 ;
	virtual void	intersect( FollowTerrainAction *, float, float ) = 0 ;
	virtual SbBool	checkObstruction( SbVec3f &localStart,
				SbVec3f &localEnd, SbBool findDetail ) = 0 ;
	static void somethingChangedCB( void *data, SoSensor *sensor ) ;
	static void rotationChangedCB( void *data, SoSensor *sensor ) ;

	float			deltaW ;
	float			deltaL ;
	float			rotCos ;
	float			rotSin ;
	SoSeparator		*root ;
	SoShapeHints		*hints ;
	SoCoordinate3		*coords ;
	SoTextureCoordinate2	*tex ;
	SoTransform		*xform ;
	SoFieldSensor		*rotationChangedSensor ;
	SoFieldSensor		*widthChangedSensor ;
	SoFieldSensor		*lengthChangedSensor ;
	SoFieldSensor		*heightChangedSensor ;

    private :

	SoChildList	children ;
} ;
#endif /* !_GROUNDSHAPE_H_ */
