//------------------------------------------------------------------------------
// InterComm class header.
//------------------------------------------------------------------------------

#ifndef	_INTERCOMM_H_
#define	_INTERCOMM_H_

#include <dmedia/audio.h>
#include "GamePackets.h"


class GameIO ;
class SfxData ;
class AudioComp ;
class CommHost ;


class CommPacket : public GamePacket {
    public :
	enum { COMP_DATA_SIZE = 1004 } ;
	CommPacket( int targetAudience, uint32_t seqId ) :
		GamePacket( COMM_PKT ),
		audience( targetAudience ),
		number( seqId ),
		len( 0 )
		{}
	signed char *getDataPtr() { return( compressedData ) ; }
	uint32_t getSequenceId() { return( number ) ; }
	int getLength() { return( len ) ; }
	int		audience ;
	int		len ;
	uint32_t	number ;
	signed char	compressedData[COMP_DATA_SIZE] ;
} ;



class CommHostList : public SbPList {

    public :
	CommHostList() : SbPList() {}
	~CommHostList() { truncate( 0 ) ; }
	CommHost *operator[]( int i ) const
		{ return( (CommHost *)( (* (const SbPList *)this )[i] ) ) ; }
	void append( CommHost *host ) { SbPList::append( (void *)host ) ; }
	void truncate( int start ) ;
	int find( HostAddr remoteId ) ;
	void remove( int which ) ;
} ;



class InterComm {

    public :
	enum Mode {
		QUIET,
		PUBLIC,
		PRIVATE
		} ;
	InterComm( GameIO *gameIO ) ;
	~InterComm() ;
	void setTeam( int newTeam ) ;
	void changeMode( Mode newMode ) ;
	void startCommProcessing() ;
	void finishCommProcessing() ;

    private :
	enum { NUM_BUFFERS = 20 } ;

	GameIO		*gio ;
	SfxData		*speaker ;
	ALport		ap ;
	int		mode ;
	int		team ;
	AudioComp	*comp ;
	CommPacket	outPacket ;
	int		enabled ;
	signed char	*rawBuffer ;
	int32_t		sampsLeftToSend ;
	int32_t		rawSampleSize ;
	signed char	*incoming ;
	int		numComm[NUM_BUFFERS] ;
	int32_t		seqId ;
	CommHostList	hostList ;
	int		readBuffer ;
	int		writeBuffer ;
	int		numBuffers ;

	InterComm() ;
	InterComm( const InterComm &ic ) ;
	int		openAudioPort() ;
	void		processComm( CommPacket *pkt ) ;
	void		addToOutputBuffer( int nSamps, CommHost *host ) ;
	static void	processCommPacket( CommPacket *pkt, void *userData ) ;
} ;

#endif	/* !_INTERCOMM_H_ */
