//------------------------------------------------------------------------------
// Class header for LosAction (Line of Sight Action)
//------------------------------------------------------------------------------

#ifndef _LOSACTION_H_
#define _LOSACTION_H_

#include <Inventor/SbLinear.h>
#include <Inventor/SbBox.h>
#include <Inventor/actions/SoSubAction.h>


class SoNode ;

class LosAction : public SoAction {

	SO_ACTION_HEADER( LosAction ) ;

    public :
	enum IntersectionMode {
		FIND_CLOSEST,
		GROSS_CHECK
		} ;

	static void initClass() ;
	LosAction() ;
	virtual ~LosAction() ;
	void setRay( const SbVec3f &start, const SbVec3f &end ) ;
	void setSelf( SoNode *n ) ;
	void setTarget( SoNode *n ) ;
	void setMode( IntersectionMode m ) { mode = m ; }
	const SoPath *getObstructor() { return( obstructor ) ; }
	IntersectionMode getMode() const { return( mode ) ; }
	SbBool isObstructed() const { return( obstructed ) ; }
	const SbVec3f &getEndPoint() const { return( endPt ) ; }
	const SbVec3f &getStartPoint() const { return( startPt ) ; }

    SoEXTENDER public :
	SbBool	checkBoundingBox( const SbBox3f &box, const SbRotation &rot,
			const SbVec3f &center, float boundingRadius ) ;
	void	trimRay( const SbVec3f &newEnd ) ;
	SbBool	isSelf( SoNode *n ) const { return( n == self ) ; }
	SbBool	isTarget( SoNode *n ) const { return( n == target ) ; }
	void	setObstructed() ;

    protected :
	SbBool checkDistance( const SbVec3f &center, float radius ) ;
	virtual void beginTraversal( SoNode *node ) ;

    private :
	static void	callDoAction( SoAction *, SoNode * ) ;
	static void	obstructS( SoAction *, SoNode * ) ;
	static void	doNada( SoAction *, SoNode * ) ;

	SbVec3f		startPt ;
	SbVec3f		endPt ;
	SbVec3f		direction ;
	float		length ;
	SbBool		obstructed ;
	IntersectionMode	mode ;

	SoNode		*self ;		// Node instigating the LOS check
	SoNode		*target ;	// Target node for the LOS check
	SoPath		*obstructor ;
} ;

#endif /* !_LOSACTION_H_ */
