//------------------------------------------------------------------------------
// Missile base class.
//------------------------------------------------------------------------------
#ifndef _MISSILE_H_
#define _MISSILE_H_

#include <Inventor/misc/SoChildList.h>
#include <Inventor/nodes/SoSubNode.h>
#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoSFRotation.h>
#include "LosAction.h"

class SoAction ;
class SoState ;
class SoGetBoundingBoxAction ;
class SoGLRenderAction ;
class SoPickAction ;
class SoGetMatrixAction ;
class SoCallbackAction ;
class Vehicle ;
class SoSeparator ;

class Missile : public SoNode {

	SO_NODE_HEADER( Missile ) ;

    public:

	enum MissileStatus {
		ACTIVE = 0,
		STRUCK,
		EXPIRED,
		} ;

	enum MissileType {
		MISSILE,
		CRUISE_MISSILE,
		GUIDED_MISSILE,
		BOMB,
		MORTAR,
		CANNON,
		BAD_TYPE
		} ;

	static void 	initClass() ;

	Missile() ;
	Missile( float height ) ;

	SoSFVec3f	position ;
	SoSFRotation	rotation ;

	SbBool		isTooOld() ;
	void 		setHeight( float h ) { desiredHeight = -h ; }
	void		setOwner( SoNode *owner )
				{ los->setSelf( owner ) ; }
	MissileStatus	getStatus() { return status ; }
	float		getBuildTime() { return( buildTime.getValue() ) ; }
	float		getDamage() { return( damageAmount ) ; }
	float		getMaxAge() { return( maxAge ) ; }
	const SoPath	*getObstructor() { return( los->getObstructor() ) ; }
	void		setVelocity( const SbVec3f &v ) { velocity = v ; }
	const SbVec3f	&getVelocity() const { return( velocity ) ; }
	float		getMaxSpeed() const { return( maxSpeed ) ; }
	virtual SbBool	isGuided() const ;
	virtual MissileType	getMissileType() const ;
	virtual SoNode	*getIconGraph() ;
	virtual MissileStatus	update( float mx, float my,
					float deltaTime, SoNode *scene ) ;

    SoEXTENDER public :
	virtual void	doAction( SoAction *action ) ;
	virtual void	getBoundingBox( SoGetBoundingBoxAction *action ) ;
	virtual void	GLRender( SoGLRenderAction *action ) ;
	virtual void	pick( SoPickAction *action ) ;
	virtual void	getMatrix( SoGetMatrixAction *action ) ;
	virtual void	callback( SoCallbackAction *action ) ;
	virtual SoChildList *getChildren() const ;
	virtual void 	blowUp() ;

    protected:
	virtual	~Missile() ;
	float			maxSpeed ;
	float			maxAge ;
	SoSFFloat		buildTime ;
	SoChildList		children ;
	float			age ;
	LosAction		*los ;
	MissileStatus		status ;
	float			desiredHeight ;
	SbVec3f			velocity ;
	float			damageAmount ;
	void			actionGuts( SoState *state, SoAction *action ) ;

    private:
	void			buildCommon() ;
	static SoSeparator	*geometry ;
	static SoSeparator	*icon ;
} ;

#endif /* !_MISSILE_H_ */
