//------------------------------------------------------------------------------
// Header file for NoPassGroup class.
//------------------------------------------------------------------------------
//
// Copyright 1995, Silicon Graphics, Inc.
// All Rights Reserved.
//
// This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
// the contents of this file may not be disclosed to third parties, copied or
// duplicated in any form, in whole or in part, without the prior written
// permission of Silicon Graphics, Inc.
//
// RESTRICTED RIGHTS LEGEND:
// Use, duplication or disclosure by the Government is subject to restrictions
// as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
// and Computer Software clause at DFARS 252.227-7013, and/or in similar or
// successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
// rights reserved under the Copyright Laws of the United States.
//
#ifndef _NOPASSGROUP_H_
#define _NOPASSGROUP_H_

#include <Inventor/fields/SoSFEnum.h>
#include <Inventor/fields/SoSFNode.h>
#include <Inventor/nodes/SoGroup.h>

class NoPassGroup : public SoGroup {

	SO_NODE_HEADER( NoPassGroup ) ;

    public :
	enum ActionFlags {		// Possible values for action banning
		NONE = 0x00,		// No bans (acts like a group)
		RENDER = 0x01,		// GLRendering
		SEARCH = 0x02,		// Searching
		PICK = 0x04,		// Picking
		BOUNDINGBOX = 0x08,	// GetBoundingBox
		MATRIX = 0x10,		// GetMatrix
		ALL = 0x1f		// All of the above
	} ;

	// Fields
	SoSFEnum	banFlags ;	// Choose an ActionFlag

	SoSFNode	alternateRep ;	// Alternate representation

	NoPassGroup() ;
	NoPassGroup( int numChildren ) ;

	virtual SbBool affectsState() const ;
	static void initClass() ;

	virtual void GLRender( SoGLRenderAction *action ) ;
	virtual void getBoundingBox( SoGetBoundingBoxAction *action ) ;
	virtual void getMatrix( SoGetMatrixAction *action ) ;
	virtual void pick( SoPickAction *action ) ;
	virtual void search( SoSearchAction *action ) ;
	virtual void write( SoWriteAction *action ) ;

    protected :
	virtual	~NoPassGroup() ;

    private :
	void	commonConstructor() ;
} ;

#endif /* _NOPASSGROUP_H_ */
