//------------------------------------------------------------------------------
// Radar node.
//------------------------------------------------------------------------------

#include <GL/gl.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/elements/SoLineWidthElement.h>
#include <Inventor/elements/SoPointSizeElement.h>
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
#include <Inventor/elements/SoLazyElement.h>
#include <Inventor/elements/SoGLLazyElement.h>
#else
#include <Inventor/elements/SoLightModelElement.h>
#include <Inventor/elements/SoDiffuseColorElement.h>
#endif
#include "Radar.h"


SO_NODE_SOURCE( Radar ) ;

//------------------------------------------------------------------------------
// Initialize the class.
//------------------------------------------------------------------------------
void
Radar::initClass()
{
	SO_NODE_INIT_CLASS( Radar, SoNode, "Node" ) ;
}



//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
Radar::Radar()
{
	SO_NODE_CONSTRUCTOR( Radar ) ;
	SO_NODE_ADD_FIELD( target, ( 0.0f, 0.0f, 0.0f ) ) ;
	SO_NODE_ADD_FIELD( center, ( 0.0f, 0.0f, 0.0f ) ) ;
	SO_NODE_ADD_FIELD( rotation, ( 0.0f, 0.0f, 1.0f, 0.0f ) ) ;
	SO_NODE_ADD_FIELD( rgb, ( 1.0f, 1.0f, 1.0f ) ) ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
Radar::~Radar()
{
}



//------------------------------------------------------------------------------
// GLRender action handler.
//------------------------------------------------------------------------------
void
Radar::GLRender(
	SoGLRenderAction *action
	)
{
	SbVec3f endPt ;

	calculateUp() ;

	SoState *state = action->getState() ;
	state->push() ;

	// Turn off lighting.
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	SoLazyElement::setLightModel( state, SoLazyElement::BASE_COLOR ) ;
	SoLineWidthElement::set( state, 2.0f ) ;
	SoPointSizeElement::set( state, 5.0f ) ;
	// SoGLLazyElement::sendTop( state, SoLazyElement::NO_COLOR_MASK ) ;
	SoGLLazyElement::sendNoMaterial( state ) ;
#else
	SoLightModelElement::set( state, this,
					SoLightModelElement::BASE_COLOR ) ;
	SoLineWidthElement::set( state, this, 2.0f ) ;
	SoPointSizeElement::set( state, this, 5.0f ) ;
#endif

	glPushMatrix() ;

	SbVec3f axis ;
	float angle ;
	rotation.getValue().getValue( axis, angle ) ;
	glRotatef( angle * (-180.0f/(float)M_PI), axis[0], axis[1], axis[2] ) ;

	SbVec3f &middle = -center.getValue() ;
	glTranslatef( middle[0], middle[1], middle[2] ) ;

	int num = target.getNum() ;

	if( rgb.getNum() != num )
	{
		if( rgb.getNum() > 0 )
		{
			glColor3fv( rgb[0].getValue() ) ;
		}

		glBegin( GL_LINES ) ;
		for( int i = 0 ; i < target.getNum() ; i++ )
		{
			SbVec3f v( target[i] ) ;
			glVertex3fv( v.getValue() ) ;

			float z = up.dot( v + middle ) ;
			endPt.setValue( v[0]-z*up[0], v[1]-z*up[1],
					v[2]-z*up[2] ) ;

			glVertex3fv( endPt.getValue() ) ;
		}
		glEnd() ;

		glBegin( GL_POINTS ) ;
		for( i = 0 ; i < target.getNum() ; i++ )
		{
			glVertex3fv( target[i].getValue() ) ;
		}
		glEnd() ;
	}
	else
	{
		const SbColor *c = rgb.getValues( 0 ) ;

		glBegin( GL_LINES ) ;
		for( int i = 0 ; i < target.getNum() ; i++ )
		{
			glColor3fv( c[i].getValue() ) ;
			SbVec3f v( target[i] ) ;
			glVertex3fv( v.getValue() ) ;

			float z = up.dot( v + middle ) ;
			endPt.setValue( v[0]-z*up[0], v[1]-z*up[1],
					v[2]-z*up[2] ) ;

			glVertex3fv( endPt.getValue() ) ;
		}
		glEnd() ;

		glBegin( GL_POINTS ) ;
		for( i = 0 ; i < target.getNum() ; i++ )
		{
			glColor3fv( c[i].getValue() ) ;
			glVertex3fv( target[i].getValue() ) ;
		}
		glEnd() ;
	}

	glPopMatrix() ;

#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	SoGLLazyElement *le = SoGLLazyElement::getInstance( state ) ;
	le->reset( state, SoLazyElement::DIFFUSE_MASK ) ;
#else
	if( rgb.getNum() == 1 )
	{
		SoDiffuseColorElement::set( state, this, 1,
					rgb.getValues( 0 ) ) ;
	}
	else if( rgb.getNum() == num )
	{
		SoDiffuseColorElement::set( state, this, num,
					rgb.getValues( 0 ) ) ;
	}
#endif

	state->pop() ;
}



