//------------------------------------------------------------------------------
// Class for radar interface.
//------------------------------------------------------------------------------

#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include "Radar.h"
#include "RadarControl.h"


static const float RadarRange[4] = { 15.0f, 30.0f, 60.0f, 180.0f } ;

//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
RadarControl::RadarControl()
{
	if( Radar::getClassTypeId() == SoType::badType() )
	{
		Radar::initClass() ;
	}

	root = new SoSeparator ;
	root->ref() ;
	root->renderCaching = SoSeparator::OFF ;
	root->boundingBoxCaching = SoSeparator::OFF ;
	root->renderCulling = SoSeparator::OFF ;
	root->pickCulling = SoSeparator::OFF ;

	SoBaseColor *bc = new SoBaseColor ;
	root->addChild( bc ) ;
	bc->rgb.setValue( 0.0f, 0.0f, 0.0f ) ;

	SoSeparator *swSep = new SoSeparator ;
	root->addChild( swSep ) ;
	swSep->renderCaching = SoSeparator::OFF ;
	swSep->boundingBoxCaching = SoSeparator::OFF ;
	swSep->renderCulling = SoSeparator::OFF ;
	swSep->pickCulling = SoSeparator::OFF ;

	SoCoordinate3 *c = new SoCoordinate3 ;
	swSep->addChild( c ) ;
	c->point.setNum( 4 ) ;
	c->point.set1Value( 0, -1.25f, -0.75f, -1.01f ) ;
	c->point.set1Value( 1, -1.15f, -0.75f, -1.01f ) ;
	c->point.set1Value( 2, -1.25f, 0.75f, -1.01f ) ;
	c->point.set1Value( 3, -1.15f, 0.75f, -1.01f ) ;

	SoTriangleStripSet *ts = new SoTriangleStripSet ;
	swSep->addChild( ts ) ;
	ts->numVertices.setValue( 4 ) ;

	swSep->addChild( buildRadarGrid( 0 ) ) ;

	rangeSwitch = new SoSwitch ;
	swSep->addChild( rangeSwitch ) ;

	SoGroup *group = new SoGroup ;
	rangeSwitch->addChild( group ) ;
	group->addChild( buildRadarGrid( 1 ) ) ;
	SoTranslation *trans = new SoTranslation ;
	group->addChild( trans ) ;
	trans->translation.setValue( -1.2f, -0.75f, -0.96f ) ;

	group = new SoGroup ;
	rangeSwitch->addChild( group ) ;
	group->addChild( buildRadarGrid( 2 ) ) ;
	trans = new SoTranslation ;
	group->addChild( trans ) ;
	trans->translation.setValue( -1.2f, -0.25f, -0.96f ) ;

	group = new SoGroup ;
	rangeSwitch->addChild( group ) ;
	group->addChild( buildRadarGrid( 3 ) ) ;
	trans = new SoTranslation ;
	group->addChild( trans ) ;
	trans->translation.setValue( -1.2f, 0.25f, -0.96f ) ;

	group = new SoGroup ;
	rangeSwitch->addChild( group ) ;
	group->addChild( buildRadarGrid( 4 ) ) ;
	trans = new SoTranslation ;
	group->addChild( trans ) ;
	trans->translation.setValue( -1.2f, 0.75f, -0.96f ) ;

	c = new SoCoordinate3 ;
	swSep->addChild( c ) ;
	c->point.set1Value( 0, -.1f, -.05f, .1f ) ;
	c->point.set1Value( 1, .1f, -.05f, .1f ) ;
	c->point.set1Value( 2, -.1f, .05f, .1f ) ;
	c->point.set1Value( 3, .1f, .05f, .1f ) ;
	c->point.set1Value( 4, -.1f, -.05f, -.1f ) ;
	c->point.set1Value( 5, .1f, -.05f, -.1f ) ;
	c->point.set1Value( 6, -.1f, .05f, -.1f ) ;
	c->point.set1Value( 7, .1f, .05f, -.1f ) ;

	bc = new SoBaseColor ;
	swSep->addChild( bc ) ;
	bc->rgb.set1Value( 0, 0.05f, 0.05f, 0.1f ) ;
	bc->rgb.set1Value( 1, 0.1f, 0.1f, 0.2f ) ;
	bc->rgb.set1Value( 2, 0.075f, 0.075f, 0.15f ) ;

	SoMaterialBinding *mb = new SoMaterialBinding ;
	swSep->addChild( mb ) ;
	mb->value = SoMaterialBinding::PER_VERTEX_INDEXED ;

	SoIndexedTriangleStripSet *it = new SoIndexedTriangleStripSet ;
	swSep->addChild( it ) ;
	int32_t itci[] = { 0, 1, 2, 3, -1, 0, 1, 4, 5, -1, 7, 5, 3, 1, -1 } ;
	int32_t itmi[] = { 1, 1, 1, 1, -1, 0, 0, 0, 0, -1, 2, 2, 2, 2, -1 } ;
	it->coordIndex.setValues( 0, sizeof(itci)/sizeof(itci[0]), itci );
	it->materialIndex.setValues( 0, sizeof(itmi)/sizeof(itmi[0]), itmi );

	SoLightModel *lm = new SoLightModel ;
	root->addChild( lm ) ;
	lm->model = SoLightModel::BASE_COLOR ;

	radarScale = new SoScale ;
	root->addChild( radarScale ) ;

	bc = new SoBaseColor ;
	root->addChild( bc ) ;
	bc->rgb.setValue( 1.0f, 1.0f, 1.0f ) ;

	radarNode = new Radar ;
	root->addChild( radarNode ) ;
	radarNode->target.setNum( 0 ) ;

	setRange( RadarControl::MEDIUM ) ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
RadarControl::~RadarControl()
{
	root->unref() ;
}



//------------------------------------------------------------------------------
// Add a target to the radar.
//------------------------------------------------------------------------------
void
RadarControl::addTarget(
	const SbVec3f	&location,
	const SbColor	&c
	)
{
	int n = radarNode->target.getNum() ;
	radarNode->target.set1Value( n, location ) ;
	radarNode->rgb.set1Value( n, c ) ;
}



//------------------------------------------------------------------------------
// Attach the location and rotation of the radar to something.
//------------------------------------------------------------------------------
void
RadarControl::attach(
	SoSFVec3f	*location,
	SoSFRotation	*rotation
	)
{
	radarNode->center.connectFrom( location ) ;
	radarNode->rotation.connectFrom( rotation ) ;
}



//------------------------------------------------------------------------------
// Clear the radar of any targets.
//------------------------------------------------------------------------------
void
RadarControl::clear()
{
	radarNode->target.setNum( 0 ) ;
	radarNode->rgb.setNum( 0 ) ;
}



//------------------------------------------------------------------------------
// Decrease the radar range.
//------------------------------------------------------------------------------
SbBool
RadarControl::decreaseRange()
{
	SbBool st ;

	switch( currentRange )
	{
		default :
		case RadarControl::SHORT :
			st = FALSE ;
			break ;

		case RadarControl::MEDIUM :
			setRange( RadarControl::SHORT ) ;
			st = TRUE ;
			break ;

		case RadarControl::FAR :
			setRange( RadarControl::MEDIUM ) ;
			st = TRUE ;
			break ;

		case RadarControl::EXTRA_FAR :
			setRange( RadarControl::FAR ) ;
			st = TRUE ;
			break ;
	}

	return( st ) ;
}



//------------------------------------------------------------------------------
// Detach the Radar node's center and rotation fields.
//------------------------------------------------------------------------------
void
RadarControl::detach()
{
	radarNode->center.disconnect() ;
	radarNode->rotation.disconnect() ;
}



//------------------------------------------------------------------------------
// Increase the radar range.
//------------------------------------------------------------------------------
SbBool
RadarControl::increaseRange()
{
	SbBool st ;

	switch( currentRange )
	{
		default :
		case RadarControl::EXTRA_FAR :
			st = FALSE ;
			break ;

		case RadarControl::FAR :
			setRange( RadarControl::EXTRA_FAR ) ;
			st = TRUE ;
			break ;

		case RadarControl::MEDIUM :
			setRange( RadarControl::FAR ) ;
			st = TRUE ;
			break ;

		case RadarControl::SHORT :
			setRange( RadarControl::MEDIUM ) ;
			st = TRUE ;
			break ;
	}
	return( st ) ;
}



//------------------------------------------------------------------------------
// Determine if the location is withing radar range.
//------------------------------------------------------------------------------
SbBool
RadarControl::isInRange(
	const SbVec3f	&location
	) const
{
	SbVec3f &d = location - radarNode->center.getValue() ;
	return( d.dot( d ) <= rangeSqrd ) ;
}



//------------------------------------------------------------------------------
// Set the radar's range.
//------------------------------------------------------------------------------
void
RadarControl::setRange(
	RangeValue	value
	)
{
	float	range ;

	switch( value )
	{
		default :
		case RadarControl::SHORT :
			rangeSwitch->whichChild = 0 ;
			range = RadarRange[0] ;
			break ;

		case RadarControl::MEDIUM :
			rangeSwitch->whichChild = 1 ;
			range = RadarRange[1] ;
			break ;

		case RadarControl::FAR :
			rangeSwitch->whichChild = 2 ;
			range = RadarRange[2] ;
			break ;

		case RadarControl::EXTRA_FAR :
			rangeSwitch->whichChild = 3 ;
			range = RadarRange[3] ;
			break ;
	}

	rangeSqrd = range * range ;
	range = 1.0f / range ;
	radarScale->scaleFactor.setValue( range, range, range ) ;

	currentRange = value ;
}



//------------------------------------------------------------------------------
// Utility function to build the scene graph for the radar grid.
//------------------------------------------------------------------------------
SoNode *
RadarControl::buildRadarGrid(
	int step
	)
{
	const int nPts = 16 ;

	if( step == 0 )
	{
		SoCoordinate3 *coord = new SoCoordinate3 ;
		coord->ref() ;
		coord->point.setNum( 3 * nPts ) ;
		SbVec3f *pts = coord->point.startEditing() ;
		for( int i = 0 ; i < nPts ; i++ )
		{
			float theta = 2.0f * M_PI * (float)i / (float)nPts ;
			float c = cosf( theta ) ;
			float s = sinf( theta ) ;
			pts[i].setValue( c, s, 0.0f ) ;
			pts[i+nPts].setValue( 0.5f * c, 0.5f * s, 0.0f ) ;
			pts[i+2*nPts].setValue( 0.333f * c, 0.333f * s, 0.0f ) ;
		}
		coord->point.finishEditing() ;
		coord->unrefNoDelete() ;

		return( coord ) ;
	}
	else
	{
		SoSeparator *base = new SoSeparator ;
		base->renderCaching = SoSeparator::ON ;
		base->renderCulling = SoSeparator::OFF ;
		base->pickCulling = SoSeparator::OFF ;
		base->boundingBoxCaching = SoSeparator::ON ;
		base->ref() ;

		SoIndexedLineSet *ls = new SoIndexedLineSet ;
		base->addChild( ls ) ;
		ls->coordIndex.setNum( nPts + 2 ) ;
		int32_t *ind = ls->coordIndex.startEditing() ;
		int32_t n ;
		for( int i = 0 ; i < nPts ; i++ )
		{
			ind[i] = i ;
		}
		ind[i++] = 0 ;
		ind[i++] = -1 ;
		ls->coordIndex.finishEditing() ;
		n = i ;

		if( step > 1 )
		{
			ls->coordIndex.setNum( n + nPts + 2 ) ;
			ind = ls->coordIndex.startEditing() ;
			int32_t k = ( step == 4 ) ? 2 * nPts : nPts ;
			for( i = 0 ; i < nPts ; i++ )
			{
				ind[n+i] = k + i ;
			}
			n += i ;
			ind[n++] = k ;
			ind[n++] = -1 ;
			ls->coordIndex.finishEditing() ;
		}

		if( step < 3 )
		{
			ls->coordIndex.setNum( n + 6 ) ;
			ind = ls->coordIndex.startEditing() ;
			int j = nPts / 8 ;
			ind[n++] = 2 * j ;
			ind[n++] = 6 * j ;
			ind[n++] = -1 ;
			ind[n++] = 0 ;
			ind[n++] = 4 * j ;
			ind[n++] = -1 ;
			ls->coordIndex.finishEditing() ;
		}
		else
		{
			ls->coordIndex.setNum( n + 9 ) ;
			ind = ls->coordIndex.startEditing() ;
			int j = nPts / 8 ;
			ind[n++] = 2 * j ;
			ind[n++] = 6 * j ;
			ind[n++] = -1 ;
			ind[n++] = j ;
			ind[n++] = 5 * j ;
			ind[n++] = 3 * j ;
			ind[n++] = 7 * j ;
			ind[n++] = j ;
			ind[n++] = -1 ;
			ls->coordIndex.finishEditing() ;
		}

		base->unrefNoDelete() ;
		return( base ) ;
	}
}
