//------------------------------------------------------------------------------
// RadarControl class header.  Provides an interface for the Radar node.
//------------------------------------------------------------------------------

#ifndef	_RADARCONTROL_H_
#define	_RADARCONTROL_H_

#include <Inventor/SbBasic.h>

class SoScale ;
class SoSeparator ;
class SoSwitch ;
class Radar ;
class SoNode ;
class SoSFVec3f ;
class SoSFRotation ;

class RadarControl {

    public :
	enum RangeValue {
		SHORT,
		MEDIUM,
		FAR,
		EXTRA_FAR
		} ;

	RadarControl() ;
	~RadarControl() ;

	void	addTarget( const SbVec3f &location, const SbColor &c ) ;
	void	attach( SoSFVec3f *location, SoSFRotation *rotation ) ;
	void	clear() ;
	SbBool	decreaseRange() ;
	void	detach() ;
	SoNode	*getSceneGraph() const { return( (SoNode *)root ) ; }
	SbBool	increaseRange() ;
	SbBool	isInRange( const SbVec3f &location ) const ;
	void	setRange( RangeValue value ) ;

    private :
	SoSeparator	*root ;
	Radar		*radarNode ;
	SoSwitch	*rangeSwitch ;
	SoScale		*radarScale ;
	float		rangeSqrd ;
	RangeValue	currentRange ;
	SoNode		*buildRadarGrid( int step ) ;
} ;

#endif	/* _RADARCONTROL_H_ */
