//------------------------------------------------------------------------------
// SceneryFile class.
//------------------------------------------------------------------------------

#include <Inventor/SoDB.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/errors/SoReadError.h>
#include "SceneryFile.h"

SO_NODE_SOURCE( SceneryFile ) ;



//------------------------------------------------------------------------------
// Initialize the class.
//------------------------------------------------------------------------------
void
SceneryFile::initClass()
{
	SO_NODE_INIT_CLASS( SceneryFile, SoGroup, "Group" ) ;
}



//------------------------------------------------------------------------------
// Common constructor code.
//------------------------------------------------------------------------------
void
SceneryFile::createIt()
{
	SO_NODE_CONSTRUCTOR( SceneryFile ) ;
	SO_NODE_ADD_FIELD( name, ( "<Undefined file>" ) ) ;
}



//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
SceneryFile::SceneryFile()
{
	createIt() ;
}



//------------------------------------------------------------------------------
// Alternate constructor.
//------------------------------------------------------------------------------
SceneryFile::SceneryFile(
	const char *s
	)
{
	createIt() ;
	name.setValue( s ) ;
	readGeometry() ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
SceneryFile::~SceneryFile()
{
}



//------------------------------------------------------------------------------
// Read in the geometry.
//------------------------------------------------------------------------------
SbBool
SceneryFile::readGeometry()
{
	SoInput in ;

	removeAllChildren() ;

	if( name.isDefault() )
	{
		SoError::post( "ScenerFile name field not set." ) ;
		return( FALSE ) ;
	}

	const char *filename = name.getValue().getString() ;

	if( !in.openFile( filename, TRUE ) )
	{
		SoReadError::post( &in, "Can't open included file \"%s\" in "
			"SceneryFile node", filename ) ;
		return( FALSE ) ;
	}

	SoNode *child ;
	SbBool readOK = FALSE ;
	while( SoDB::read( &in, child ) )
	{
		if( child == NULL )
		{
			readOK = TRUE ;
			break ;
		}
		addChild( child ) ;
	}

	in.closeFile() ;

	if( !readOK )
	{
		SoReadError::post( &in, "Bad data in \"%s\"", filename ) ;
	}

	return( readOK ) ;
}



//------------------------------------------------------------------------------
// Read in an instance of the node.
//------------------------------------------------------------------------------
SbBool
SceneryFile::readInstance(
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	SoInput		*in,
	unsigned short	flags
#else
	SoInput	*in
#endif
	)
{
	SbBool status = TRUE ;

	// Read field info as usual.
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	if( !SoFieldContainer::readInstance( in, flags ) )
#else
	if( !SoFieldContainer::readInstance( in ) )
#endif
	{
		status = FALSE ;
	}
	else
	{
		status = readGeometry() ;
	}

	return( status ) ;
}



//------------------------------------------------------------------------------
// Write action handler.
//------------------------------------------------------------------------------
void
SceneryFile::write(
	SoWriteAction *action
	)
{
	SoNode::write( action ) ;
}
