//------------------------------------------------------------------------------
// Scout class.
//------------------------------------------------------------------------------

#include <Inventor/fields/SoMFString.h>

#include "Missile.h"
#include "GuidedMissile.h"
#include "Scout.h"


SO_NODE_SOURCE( Scout ) ;



//------------------------------------------------------------------------------
// Initialize the Scout class.
//------------------------------------------------------------------------------
void
Scout::initClass()
{
	if( Tank::getClassTypeId() == SoType::badType() )
	{
		Tank::initClass() ;
	}
	SO_NODE_INIT_CLASS( Scout, Tank, "Tank" ) ;
}



//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
Scout::Scout() :
	Tank( "models/scout.iv" )
{
	createIt() ;
}



//------------------------------------------------------------------------------
// Alternate constructor.
//------------------------------------------------------------------------------
Scout::Scout(
	const char *name
	) :
	Tank( name )
{
	createIt() ;
}



//------------------------------------------------------------------------------
// Common constructor code.
//------------------------------------------------------------------------------
void
Scout::createIt()
{
	SO_NODE_CONSTRUCTOR( Scout ) ;

	Scout::setParams() ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
Scout::~Scout()
{
}



//------------------------------------------------------------------------------
// Set up parameters.
//------------------------------------------------------------------------------
void
Scout::setParams()
{
	maxSpeed = 3.0f ;
	maxTurnRate = 1.5f ;
	maxAcceleration = 4.0f ;
	maxDamage = 8.0f ;
	numBays = 2 ;

	numWeapons[WeaponControl::MAIN_POD] = 2 ;
	weaponList[WeaponControl::MAIN_POD].truncate( 0 ) ;
	weaponList[WeaponControl::MAIN_POD].append( Missile::MISSILE ) ;
	weaponList[WeaponControl::MAIN_POD].append( Missile::GUIDED_MISSILE ) ;
	numWeapons[WeaponControl::AA_POD] = 0 ;
	weaponList[WeaponControl::AA_POD].truncate( 0 ) ;

	modelLabel = "Scout" ;

	points = 40 ;
}



//------------------------------------------------------------------------------
// Indicate that no con view is available.
//------------------------------------------------------------------------------
SbBool
Scout::hasConView() const
{
	return( FALSE ) ;
}



//------------------------------------------------------------------------------
// Show vehicle-specific help text.
//------------------------------------------------------------------------------
void
Scout::showHelp(
	SoMFString	*str,
	int		&n
	)
{
	str->set1Value( n++, "Left mouse - fire next missile" ) ;
	str->set1Value( n++, "Middle mouse - change missile type to be "
				"loaded" ) ;
	str->set1Value( n++, "Mouse - control speed and direction" ) ;
}
