//------------------------------------------------------------------------------
// SfxPoint -- audio sound effects class header.
//
// Chris Fouts - January 1995.
//
// $Id: SfxPoint.h,v 1.4 1995/07/18 01:54:30 fouts Exp $
//------------------------------------------------------------------------------
//
// Copyright 1995, Silicon Graphics, Inc.
// All Rights Reserved.
//
// This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
// the contents of this file may not be disclosed to third parties, copied or
// duplicated in any form, in whole or in part, without the prior written
// permission of Silicon Graphics, Inc.
//
// RESTRICTED RIGHTS LEGEND:
// Use, duplication or disclosure by the Government is subject to restrictions
// as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
// and Computer Software clause at DFARS 252.227-7013, and/or in similar or
// successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
// rights reserved under the Copyright Laws of the United States.
//

#ifndef	_SFXPOINT_H
#define	_SFXPOINT_H

#include <audio.h>
#include <signal.h>
#include "Sfx.h"

class SfxPoint : public Sfx {
    public:

	// Constructors
	SfxPoint( const char *fileName ) ;
	SfxPoint( SfxPoint *s ) ;
	SfxPoint( SfxPoint &s ) ;

	// Destructor
	virtual ~SfxPoint() ;

	// Set the distance at which the volume will be halved.
	void		setHalfVolumeDistance( float d ) ;

	// Return the distance at which the volume is half the original.
	float		getHalfVolumeDistance() ;

	// Set the location of the point sound source.
	void		setLocation( float x, float y = 0.0f, float z = 0.0f ) ;

	// Play the sound.
	virtual void	play() ;

	// Play the sound at a new location.
	void		play( float x, float y = 0.0f, float z = 0.0f ) ;

    protected:
	int		leftNum ;	// numerator for left volume
	int		rightNum ;	// numerator for left volume
	int		denominator ;	// denominator for volume
	float		distanceScale ;	// scale value for distances
	virtual int	fillQueue( int * ) ;
    } ;

#endif /* _SFXPOINT_H */
