//------------------------------------------------------------------------------
// Tank class header.
//------------------------------------------------------------------------------
#ifndef _TANK_H_
#define _TANK_H_

#include "Vehicle.h"

class FollowTerrainAction ;
class CollisionAction ;
class GroundSpot ;
class SoCoordinate3 ;
class SoIndexedLineSet ;

class Tank : public Vehicle {

	SO_NODE_HEADER( Tank ) ;

    public :
	static void	initClass() ;
	Tank() ;
	Tank( const char *name ) ;
	virtual void	reset() ;
	virtual void	showHelp( SoMFString *str, int &nLines ) ;

    SoEXTENDER public :
	virtual float	move( float xInput, float yInput, float deltaTime,
		SoNode *scene ) ;
	SbVec3f		omegaAxis ;
	float		omegaRad ;
	float		width ;
	float		height ;
	float		length ;
	float		mass ;
	float		Ix ;
	float		Iy ;
	float		Iz ;

    protected :
	virtual SbBool	followTerrain( SoNode *terrain, SbVec3f &pos,
				SbMatrix &mat, SbVec3f &collisionNormal,
				float deltaTime ) ;
	float findEnergy( const GroundSpot *spot, int i, int j, int k,
			int nPoints, const SbVec3f &pos, const SbMatrix &mat,
			float deltaTime, SbBool &flip ) ;
	void findPosition( const GroundSpot &a, const GroundSpot &b,
			const GroundSpot &c, SbVec3f &pos, SbMatrix &mat ) ;
	SbRotation &findRotation( const SbMatrix &start, const SbMatrix &end ) ;
	float calculateEnergy( const SbVec3f &realCenterPos,
		const SbRotation &realRot, const SbVec3f &freeCenterPos,
		const SbRotation &freeRot, float deltaTime ) const ;
	virtual void	boundIt() ;
	virtual		~Tank() ;

	SbVec3f	gpUp ;
	SbVec3f	gpRt ;
	SbVec3f	gpAh ;
	float	gpW ;
	float	gpL ;
	float	gpH ;
	int	gpMode ;
	void	findBottomSide( const SbMatrix &mat ) ;
	float	findTwist( const SbRotation &r, const SbVec3f &u,
			const SbVec3f &v ) ;
	void	calculateOmega( SbRotation &newOrient, SbRotation &omega,
			float desiredTwist ) ;

    private :
	void createIt() ;
	void smoothRide( SbMatrix &mat ) ;
	SoCoordinate3 *dbCoord ;
	SoIndexedLineSet *dbLine ;
	SoCoordinate3 *dbFreeCoord ;
} ;

#endif /* !_TANK_H_ */
