//------------------------------------------------------------------------------
// Vehicle abstract class header.
//------------------------------------------------------------------------------
#ifndef _VEHICLE_H_
#define _VEHICLE_H_

#include <Inventor/SbLinear.h>
#include <Inventor/SoLists.h>
#include <Inventor/fields/SoSFBool.h>
#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFRotation.h>
#include <Inventor/fields/SoSFString.h>
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/nodes/SoSubNode.h>
#include <X11/X.h>
#include "WeaponControl.h"
#include "Missile.h"
#include "Obstacle.h"

class FollowTerrainAction ;
class CollisionAction ;
class LosAction ;
class SoSwitch ;
class SoMFString ;


class MissileTypeList : public SbPList {
    public :
	MissileTypeList() : SbPList() {}
	~MissileTypeList() { truncate( 0 ) ; }
	Missile::MissileType operator[]( int i ) const
	    { return( (Missile::MissileType)((*(const SbPList *)this )[i])) ; }
	void append( Missile::MissileType type )
	    { SbPList::append( (void *)type ) ; }
	void insert( Missile::MissileType type, int addBefore )
	    { SbPList::insert( (void *)type, addBefore ) ; }
	void remove( int which ) { SbPList::remove( which ) ; }
	void truncate( int start ) { SbPList::truncate( start ) ; }
} ;



class Vehicle : public Obstacle {

	SO_NODE_HEADER( Vehicle ) ;

    public :
	SoSFRotation	radarRotation ;
	static void	initClass() ;
	Vehicle() ;
	Vehicle( const char *name ) ;
	virtual float	move( float omega, float accel, float deltaTime,
				SoNode *scene ) ;
	virtual float	moveConView( float mx, float my, float dt,
				SoNode *scene ) ;
	void		resetConView() ;
	virtual void	reset() {}
	SoSFRotation	conView ;
	float		getMaxDamage() { return( maxDamage ) ; }
	int		getNumBays() { return( numBays ) ; }
	virtual SbBool	hasConView() const ;
	virtual SbBool	hasRangeFinder() const ;
	virtual void	setRange( float ) {}
	void		getEngineSpeeds( float &left, float &right )
				{ left = leftEngine ; right = rightEngine ; }
	int		getNumWeapons( WeaponControl::WeaponPod pod ) ;
	Missile::MissileType getWeaponType( int n,
				WeaponControl::WeaponPod pod ) ;
	const SbString	&getLabel() const { return( modelLabel ) ; }
	const SbVec3f	&getMainViewpoint() const { return( mainViewpoint ) ; }
	const SbVec3f	&getAuxViewpoint() const { return( auxViewpoint ) ; }
	const SbVec3f	&getVelocity() { return( velocity ) ; }
	float		getMaxSpeed() { return( maxSpeed ) ; }
	virtual SbBool	handleKeyPress( KeySym ks ) ;
	virtual void	launchWeapon( Missile *weapon,
				WeaponControl::WeaponPod pod ) ;
	WeaponControl::WeaponPod	getBayPod() const { return( bayPod ) ; }
	void		showFirstPersonGraphics( SbBool show,
				WeaponControl::WeaponPod ) ;
	int		getPoints() { return( points ) ; }
	virtual SbBool	okayToFire( WeaponControl::WeaponPod pod ) ;
	virtual void	showHelp( SoMFString *str, int &nLines ) ;

    SoEXTENDER public :
	virtual void	collide( CollisionAction *action ) ;
	virtual void	obstruct( LosAction *action ) ;

    protected :
	SbString	modelLabel ;
	virtual void	processGeometry() ;
	virtual SbBool	followTerrain( SoNode *terrain, SbVec3f &pos,
				SbMatrix &mat, SbVec3f &collisionNormal,
				float deltaTime ) ;
	virtual void	boundIt() ;
	void		setParams() ;
	float		maxTurnRate ;
	float		maxAcceleration ;
	float		maxSpeed ;
	float		speed ;
	FollowTerrainAction *fta ;
	CollisionAction *ca ;
	SbVec3f	 velocity ;
	float heightClearance ;
	int onGround ;
	virtual ~Vehicle() ;
	float maxDamage ;
	float numBays ;
	float leftEngine ;
	float rightEngine ;
	int   numWeapons[WeaponControl::MAX_PODS] ;
	MissileTypeList	weaponList[WeaponControl::MAX_PODS] ;
	WeaponControl::WeaponPod	bayPod ;
	SbVec3f	mainViewpoint ;
	SbVec3f auxViewpoint ;
	SbVec3f launchPoint ;
	SoSwitch	*fpGraphics ;
	float	azim ;
	float	incl ;
	int	points ;

    private :
	void createIt() ;
} ;

#endif /* !_VEHICLE_H_ */
