//------------------------------------------------------------------------------
// Header for WeaponControl class.
//------------------------------------------------------------------------------

#ifndef	_WEAPONCONTROL_H_
#define	_WEAPONCONTROL_H_

#include <Inventor/SbBasic.h>
#include <Inventor/SoType.h>
#include <Inventor/SbColor.h>
#include "GamePackets.h"

class SoBaseColor ;
class SoGroup ;
class SoNode ;
class SoSeparator ;
class SoSwitch ;
class WeaponBay ;
class Cannon ;
class BigGun ;
class Missile ;
class ExplosionControl ;
class Sfx ;

class MissilePack {
    public :
	MissilePack() ;
	virtual ~MissilePack() ;
	void init( Missile *m ) ;
	void explode( HostAddr newStatus ) ;
	HostAddr getStatus() const { return( status ) ; }
	Missile *getMissile() { return( missile ) ; }
	float getDamage() const { return( damage ) ; }
	float age( float dt ) { explosionAge += dt ; return( explosionAge ) ; }

    private :
	Missile			*missile ;
	HostAddr		status ;
	float			damage ;
	float			explosionAge ;
} ;


typedef HostAddr WeaponControlStrikeHandler( SoNode *splat, Missile *mis,
						void *userData ) ;

class WeaponControl {

    public :
	enum WeaponPod {
		MAIN_POD = 0,
		AA_POD = 1,
		MAX_PODS = 2
		} ;
	enum { MAX_BAYS = Constants::MAX_BAYS } ;
	enum { MAX_BULLETS = Constants::MAX_BULLETS } ;
	enum { MAX_WEAPONS = MAX_BAYS + MAX_BULLETS } ;
	WeaponControl( int numberOfBays = MAX_BAYS ) ;
	~WeaponControl() ;

	void	setNumberOfBays( int newNumber ) ;
	void	clearWeaponTypes() ;
	Missile	*launchWeapon( WeaponPod currentPod ) ;
	Missile *launchPlayerMissile( Missile::MissileType type ) ;
	Missile *launchAutomatonMissile( SoType type ) ;
	void	terminatePlayerMissile( Missile *mis )
			{ terminateAnyonesMissile( mis ) ; }
	SbBool	changeWeapon( WeaponPod currentPod ) ;
	void	updateWeapons( float mx, float my, float deltaTime,
				SbBool updateBays, SoNode *scene ) ;
	SoGroup	*getWeaponGroup() { return( weaponGroup ) ; }
	SbBool	addWeaponType( Missile::MissileType weaponClassTypeId,
				WeaponPod view );
	SoGroup *getWeaponDisplay() { return( (SoGroup *)weaponDisplay ) ; }
	SbBool	isGuidingWeapon() { return( guidedMissile != NULL ) ; }
	Missile	*getGuidedWeapon() { return( guidedMissile ) ; }
	SbBool	isBomb( Missile::MissileType type ) const ;
	const char *getWeaponName( Missile::MissileType type ) const ;
	void	stopGuidingWeapon() { guidedMissile = NULL ; }
	SbBool	isGuidedWeaponReady( WeaponPod pod ) ;
	void	setExplosionControl( ExplosionControl *expCon )
		    { explosionControl = expCon ; }
	void	reset() ;
	void	loadMissileInfo( MissileInfo *mInfo, float dt ) ;
	void	setMissileStrikeHandler( WeaponControlStrikeHandler *func,
			void *userData ) { missileStrikeHandler = func ;
			missileStrikeData = userData ; }
	void	setBayPod( WeaponPod pod ) ;
	static const HostAddr	ActiveMissile ;
	static const HostAddr	DeadMissile ;

    private :
	void	createWeaponDisplay() ;
	void	terminateMissile( Missile *mis, HostAddr newStatus ) ;
	void	terminateAnyonesMissile( Missile *mis ) ;
	Missile	*createMissile( WeaponPod pod, int n ) ;

	SoType			weaponClassType[7] ;
	SoGroup			*weaponGroup ;
	SoGroup			*myWeaponGroup ;
	SoSeparator		*weaponDisplay ;
	int			nextBay ;
	int			nextBullet ;
	int			nextLoadingBay ;
	int			nextMainWeapon ;
	int			nextAAWeapon ;
	int			numBays ;
	Missile			*guidedMissile ;
	MissilePack		myMissile[MAX_BAYS] ;
	MissilePack		myBullet[MAX_BULLETS] ;
	WeaponBay		*bay ;
	BigGun			*cannon ;
	SoGroup			*mainWeapons ;
	SoGroup			*AAWeapons ;
	SoSwitch		*baySwitch ;
	SoBaseColor		*bayStatus[MAX_BAYS] ;
	SoBaseColor		*mainStatus ;
	SoBaseColor		*auxStatus ;
	SoBaseColor		*cannonStatus ;
	SbColor			statusColor[4] ;
	SoSeparator		*bayIcon[MAX_BAYS] ;
	SoSeparator		*nextMainWeaponGroup ;
	SoSeparator		*nextAAWeaponGroup ;
	ExplosionControl	*explosionControl ;
	WeaponPod		bayPod ;
	WeaponControlStrikeHandler	*missileStrikeHandler ;
	void				*missileStrikeData ;
	Sfx			*missileLoading ;
	Sfx			*missileReady ;
	Sfx			*fireMissile ;
	Sfx			*fireCannon ;
	Sfx			*cannonReady ;
} ;

#endif	/* _WEAPONCONTROL_H_ */
