//------------------------------------------------------------------------------
// Node to control the zbuffer function in GL.
//------------------------------------------------------------------------------

#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/actions/SoCallbackAction.h>
#include "ZbufferMode.h"


SO_NODE_SOURCE( ZbufferMode ) ;



//------------------------------------------------------------------------------
// Initialize the class.
//------------------------------------------------------------------------------
void
ZbufferMode::initClass()
{
	if( ZbufferModeElement::getClassTypeId() == SoType::badType() )
	{
		ZbufferModeElement::initClass() ;
	}
	SO_NODE_INIT_CLASS( ZbufferMode, SoNode, "Node" ) ; 
	SO_ENABLE( SoGLRenderAction, ZbufferModeElement ) ;
	SO_ENABLE( SoCallbackAction, ZbufferModeElement ) ;
}



//------------------------------------------------------------------------------
// Constructor.
//------------------------------------------------------------------------------
ZbufferMode::ZbufferMode()
{
	SO_NODE_CONSTRUCTOR( ZbufferMode ) ;

	SO_NODE_ADD_FIELD( value, ( ZbufferModeElement::getDefault() ) ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, NEVER ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, LESS ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, EQUAL ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, LEQUAL ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, GREATER ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, NOTEQUAL ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, GEQUAL ) ;
	SO_NODE_DEFINE_ENUM_VALUE( Value, ALWAYS ) ;

	SO_NODE_SET_SF_ENUM_TYPE( value, Value ) ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
ZbufferMode::~ZbufferMode()
{
}



//------------------------------------------------------------------------------
// Handle generic actions.
//------------------------------------------------------------------------------
void
ZbufferMode::doAction(
	SoAction	*action
	)
{
	SoState	*state = action->getState() ;

	if( !value.isIgnored() )
	{
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
		ZbufferModeElement::set( state,
			(ZbufferModeElement::Value)value.getValue() ) ;
#else
		ZbufferModeElement::set( state, this,
			(ZbufferModeElement::Value)value.getValue() ) ;
#endif
	}
}



//------------------------------------------------------------------------------
// Handles the callback action.
//------------------------------------------------------------------------------
void
ZbufferMode::callback(
	SoCallbackAction *action
	)
{
	ZbufferMode::doAction( action ) ;
}



//------------------------------------------------------------------------------
// Handles the GLRender action.
//------------------------------------------------------------------------------
void
ZbufferMode::GLRender(
	SoGLRenderAction *action
	)
{
	ZbufferMode::doAction( action ) ;
}
