//------------------------------------------------------------------------------
// Element for controlling the Zbuffer function.
//------------------------------------------------------------------------------

#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include "ZbufferModeElement.h"


SO_ELEMENT_SOURCE( ZbufferModeElement ) ;



//------------------------------------------------------------------------------
// Initialize the class.
//------------------------------------------------------------------------------
void
ZbufferModeElement::initClass()
{
	SO_ELEMENT_INIT_CLASS( ZbufferModeElement, SoInt32Element ) ;
	SO_ENABLE( SoGLRenderAction, ZbufferModeElement ) ;
	SO_ENABLE( SoCallbackAction, ZbufferModeElement ) ;
}



//------------------------------------------------------------------------------
// Destructor.
//------------------------------------------------------------------------------
ZbufferModeElement::~ZbufferModeElement()
{
}



//------------------------------------------------------------------------------
// Initialize the element.
//------------------------------------------------------------------------------
void
ZbufferModeElement::init(
	SoState *
	)
{
	data = getDefault() ;

	copiedFromParent = NULL ;
}



//------------------------------------------------------------------------------
// Prints the element for debugging.
//------------------------------------------------------------------------------
void
ZbufferModeElement::print(
	FILE	*fp
	) const
{
	const char *valueName ;

	SoElement::print( fp ) ;

	switch( (Value)data )
	{
		case NEVER :
			valueName = "NEVER" ;
			break ;
		case LESS :
			valueName = "LESS" ;
			break ;
		case EQUAL :
			valueName = "EQUAL" ;
			break ;
		case LEQUAL :
			valueName = "LEQUAL" ;
			break ;
		case GREATER :
			valueName = "GREATER" ;
			break ;
		case NOTEQUAL :
			valueName = "NOTEQUAL" ;
			break ;
		case GEQUAL :
			valueName = "GEQUAL" ;
			break ;
		case ALWAYS :
			valueName = "ALWAYS" ;
			break ;
	}

	fprintf( fp, "\tZbuffer value = %s\n", valueName ) ;
}



//------------------------------------------------------------------------------
// Push method to copy draw style.
//------------------------------------------------------------------------------
void
ZbufferModeElement::push(
	SoState	*state
	)
{
	const ZbufferModeElement *prevElt = 
		(const ZbufferModeElement *)getNextInStack() ;

	data = prevElt->data ;

	copiedFromParent = state ;
}



//------------------------------------------------------------------------------
// Pops element, causing side effects in GL.
//------------------------------------------------------------------------------
void
ZbufferModeElement::pop(
	SoState		*state,
	const SoElement	*childElt
	)
{
	// Make sure open caches capture this element.
	capture( state ) ;
	copiedFromParent = NULL ;

	const ZbufferModeElement *child = (const ZbufferModeElement *)childElt ;

	if( data != child->data )
	{
		send() ;
	}
}



//------------------------------------------------------------------------------
// Sets the zbuffer mode in the element.
//------------------------------------------------------------------------------
void
ZbufferModeElement::setElt(
	int32_t	value
	)
{
	// Optimization:  on push, we copy the value from the previous
	// element.  If the element is set to the same value, we don't
	// bother sending it, but do add a cache dependency on the
	// previous element.
	if( data != value )
	{
		data = value ;
		send() ;
		copiedFromParent = NULL ;
	}
	else if( copiedFromParent )
	{
		ZbufferModeElement *parent =
				(ZbufferModeElement *)getNextInStack() ;
		parent->capture( copiedFromParent ) ;
	}
}



//------------------------------------------------------------------------------
// Sets the zbuffer mode in GL.
//------------------------------------------------------------------------------
void
ZbufferModeElement::send()
{
	glDepthFunc( (GLenum)data ) ;
}
