//------------------------------------------------------------------------------
// Element that contains the current Z-buffer setting.
//------------------------------------------------------------------------------

#ifndef _ZBUFFERMODEELEMENT_H_
#define _ZBUFFERMODEELEMENT_H_

#include <Inventor/elements/SoInt32Element.h>
#include <GL/gl.h>

class ZbufferModeElement : public SoInt32Element {

	SO_ELEMENT_HEADER( ZbufferModeElement ) ;

    public :
	enum Value {
		NEVER = GL_NEVER,
		LESS = GL_LESS,
		EQUAL = GL_EQUAL,
		LEQUAL = GL_LEQUAL,
		GREATER = GL_GREATER,
		NOTEQUAL = GL_NOTEQUAL,
		GEQUAL = GL_GEQUAL,
		ALWAYS = GL_ALWAYS
		} ;

	virtual void	init( SoState *state ) ;
#if defined(SO_VERSION) && SO_VERSION > 1 && SO_VERSION_REVISION > 0
	static void	set( SoState *state, Value value )
	    { SoInt32Element::set( classStackIndex, state, (int32_t)value ); }
#else
	static void	set( SoState *state, SoNode *node, Value value )
	    { SoInt32Element::set( classStackIndex, state, node, (int32_t)value ); }
#endif
	static Value	get( SoState *state )
	    { return (Value)SoInt32Element::get( classStackIndex, state ) ; }
	static Value	getDefault() { return LESS ; }
	virtual void	print( FILE *fp ) const ;
	static void	initClass() ;

	virtual void	push( SoState *state ) ;
	virtual void	pop( SoState *state, const SoElement *prevTopElement ) ;

    protected :
	virtual ~ZbufferModeElement() ;
	virtual void	setElt( int32_t value ) ;
	SoState		*copiedFromParent ;
	void		send() ;
} ;

#endif	/* _ZBUFFERMODEELEMENT_H_ */
