/*
 * Copyright (C) 1990,91,92,93,94   Silicon Graphics, Inc.
 *
 _______________________________________________________________________
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 |
 |   $Revision: 2.2 $
 |
 |   Classes:
 |	Sky
 |
 |   Author(s)	: Dave Immel, Gavin Bell
 |
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 _______________________________________________________________________
 */

#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/elements/SoLightModelElement.h>
#include <Inventor/elements/SoViewVolumeElement.h>
#include <Inventor/elements/SoGLLazyElement.h>
//#include <Inventor/elements/SoLazyElement.h>
#include <GL/gl.h>

#include "Sky.h"

SO_NODE_SOURCE(Sky);

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Init Sky
//

void
Sky::initClass()
//
////////////////////////////////////////////////////////////////////////
{
    SO_NODE_INIT_CLASS(Sky, SoNode, "Node");
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Constructor; setup fields, etc.
//

Sky::Sky()
//
////////////////////////////////////////////////////////////////////////
{
    SO_NODE_CONSTRUCTOR(Sky);

    SO_NODE_ADD_FIELD(horizonColor, (.8, .8, 1.0));
    SO_NODE_ADD_FIELD(horizonHeight, (100.0));
    SO_NODE_ADD_FIELD(skyBottomColor, (0.0, 0.4, 0.7));
    SO_NODE_ADD_FIELD(skyTopColor, (0.0, 0.0, 0.44));
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Destructor
//

Sky::~Sky()
//
////////////////////////////////////////////////////////////////////////
{
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Render
//

void
Sky::GLRender(SoGLRenderAction *action)
//
////////////////////////////////////////////////////////////////////////
{
    SoState *state = action->getState();
    state->push();

    // Turn off lighting
/*    SoLightModelElement::set(state, this,
			     SoLightModelElement::BASE_COLOR);
*/
    SoGLLazyElement *lazyElt
          = (SoGLLazyElement*)SoLazyElement::getInstance(state);
    SoLazyElement::setLightModel(state,SoLazyElement::BASE_COLOR);
    // Don't z-buffer
    glDisable(GL_DEPTH_TEST);
    
    // Get the view volume
    const SbViewVolume &vv = SoViewVolumeElement::get(state);
    
    // Figure out stuff:
    const SbVec3f &eye = vv.getProjectionPoint();
    const SbVec3f &eyeDir = vv.getProjectionDirection();

    // Point in the middle of the view volume:
    SbVec3f midVolume = eye + eyeDir*(vv.getNearDist() +
				      vv.getDepth()*.98);
    SbVec3f midVolume0(midVolume[0], 0.0, midVolume[2]);
    
    // We'll construct the sky polygons in world space, located at
    // mid-view volume, based at y==0, oriented appropriately in x/z:

    // ??? This will not work for arbitrary eye orientations!  In
    // particular, looking straight up or down the Y axis will cause
    // problems.
    SbVec3f perpendicular(-eyeDir[2], 0.0, eyeDir[0]);
    perpendicular.normalize();
    
    const float bigWidth = 10000.0;

    // Just draw big polygons a good distance along perpendicular,
    // from the midVolume point:
    SbVec3f leftHorizon, rightHorizon;
    SbVec3f leftBottomSky, rightBottomSky;
    SbVec3f leftTopSky, rightTopSky;
    
    leftHorizon = midVolume0 + perpendicular*bigWidth;
    rightHorizon = midVolume0 - perpendicular*bigWidth;
    leftBottomSky = leftHorizon +
	SbVec3f(0.0, horizonHeight.getValue(), 0.0);
    rightBottomSky = rightHorizon +
	SbVec3f(0.0, horizonHeight.getValue(), 0.0);
    
    SoGLLazyElement::sendNoMaterial(state);
    glBegin(GL_QUADS);
    glColor3fv(horizonColor.getValue().getValue());
    glVertex3fv(leftHorizon.getValue());
    glVertex3fv(rightHorizon.getValue());
    glColor3fv(skyBottomColor.getValue().getValue());
    glVertex3fv(rightBottomSky.getValue());
    glVertex3fv(leftBottomSky.getValue());
    
//    leftTopSky = leftHorizon + SbVec3f(0.0, bigWidth, 0.0);
//    rightTopSky = rightHorizon + SbVec3f(0.0, bigWidth, 0.0);
    leftTopSky = leftHorizon + SbVec3f(0.0, bigWidth, 500.0);
    rightTopSky = rightHorizon + SbVec3f(0.0, bigWidth, 500.0);
    
    glVertex3fv(leftBottomSky.getValue());
    glVertex3fv(rightBottomSky.getValue());
    glColor3fv(skyTopColor.getValue().getValue());
    glVertex3fv(leftTopSky.getValue());
    glVertex3fv(rightTopSky.getValue());
    
    glEnd();
    
    glEnable(GL_DEPTH_TEST);

    lazyElt->reset(state, SoLazyElement::DIFFUSE_MASK);

    state->pop();
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    This node does not affect state.
//

SbBool
Sky::affectsState() const
//
////////////////////////////////////////////////////////////////////////
{
    return FALSE;
}

